package com.bull.cimero.pluginEditor.editors.model;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.spagic.utils.SpagicLogger;

import com.bull.cimero.pluginEditor.utils.propertyCombo.LabelPropertyDescriptor;

/**
 * This class is the model of a connnection beetween two Generique element.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */

public class Connection extends Element implements IPropertySource {

	/**Source CONSTANT.*/
	private static final String SOURCE = "source";
	/**destination CONSTANT.*/
	private static final String DESTINATION = "destination";
	/** serial version. */
	private static final long serialVersionUID = 1L;
	/** target of the connection. */
	private GeneriqueCimeroModel target = null;
	/** source of the connection. */
	private GeneriqueCimeroModel source = null;
	/**The source type connection. */
	private boolean sourceBottom = false;

	/**
	 * default constructor.
	 * @param sourceParam : source of the connection
	 * @param targetParam : target of the connection
	 */
	public Connection(final GeneriqueCimeroModel sourceParam,
			final GeneriqueCimeroModel targetParam) {
		this(sourceParam, targetParam, false);
	}

	/**
	 * default constructor.
	 * @param sourceParam : source of the connection
	 * @param targetParam : target of the connection
	 * @param bottomConnection : true if it is a bottom connection.
	 */
	public Connection(final GeneriqueCimeroModel sourceParam,
			final GeneriqueCimeroModel targetParam,
			final boolean bottomConnection) {
		this.sourceBottom = bottomConnection;
		this.source = sourceParam;
		this.target = targetParam;

		if (source.addOutput(this)) {
			if (!target.addInput(this)) {
				source.removeOutput(this);
			}
		}
	}

	/**
	 * function to get the target of the connection.
	 * @return GeneriqueCimeroModel : the target
	 */
	public final GeneriqueCimeroModel getTarget() {
		return this.target;
	}

	/**
	 * function to get the source of the connection.
	 * @return GeneriqueCimeroModel : the source
	 */
	public final GeneriqueCimeroModel getSource() {
		return this.source;
	}

	/**
	 * function to get the editable value.
	 * @return Object : this object
	 */
	public final Object getEditableValue() {
		return this;
	}

	/**
	 * function to get the property descriptors.
	 * @return IPropertyDescriptor : descriptors
	 */
	public final IPropertyDescriptor[] getPropertyDescriptors() {
		IPropertyDescriptor[] descr = new IPropertyDescriptor[2];
	    descr[0] = new LabelPropertyDescriptor(SOURCE, SOURCE + " : ");
	    descr[1] = new LabelPropertyDescriptor(
	    		DESTINATION, DESTINATION + " : ");
	    return descr;
	}

	/**
	 * function to get the value of an object.
	 * @param id : id of the object
	 * @return Object : value of the object
	 */
	public final Object getPropertyValue(final Object id) {
		if (SOURCE.equals(id)) {
			return source.getName();
		} else if (DESTINATION.equals(id)) {
			return target.getName();
		}
		SpagicLogger.warning("the object id doesn't exist");
		return null;
	}

	/**
	 * function to test if a property is set.
	 * @param id : id of the object to test
	 * @return boolean : test result
	 */
	public final boolean isPropertySet(final Object id) {
		return false;
	}

	/**
	 * function to reset a property value.
	 * @param id : id of the object to reset
	 */
	public final void resetPropertyValue(final Object id) {
		// TODO Auto-generated method stub
	}

	/**
	 * function to set a property value.
	 * @param id : object id
	 * @param value : object value
	 */
	public final void setPropertyValue(final Object id, final Object value) {

	}

	/**
	 * @return the sourceBottom
	 */
	public final boolean isSourceBottom() {
		return sourceBottom;
	}
}
