

package org.spagic.bpmn.conversion;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.Collections;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.spagic.utils.SpagicLogger;

/**
 * The main plugin class to be used in the desktop.
 * This classes is create by Eclipse and using
 * to manage cycle life of this plugin
 */
public class Spagic1ToSpagic2Activator extends AbstractUIPlugin  {

	// The plug-in ID
	public static final String PLUGIN_ID = "org.spagic.bpmn.conversion";
	
	/** The shared instance. */
	private static Spagic1ToSpagic2Activator plugin;


	/** The constructor. */
	public Spagic1ToSpagic2Activator() {
		plugin = this;	
	}		
				        
	/**
	 * This method is called when the plug-in is stopped.
	 * @param context : the context
	 * @throws Exception : if error
	 */
	public final void stop(final BundleContext context)
					throws Exception {
		super.stop(context);
		plugin = null;
	}

	/**
	 * Returns the shared instance.
	 * @return Spagic1ToSpagic2Activator : the plugin
	 */
	public static Spagic1ToSpagic2Activator getDefault() {
		if (plugin == null) {
			plugin = new Spagic1ToSpagic2Activator();
		}
		return plugin;
	}

	/**
	 * Returns the plugin id.
	 * @return pluginId
	 */
	public static final String getPluginId() {
		return PLUGIN_ID;
	}
	
	public static String getRessourcePath(final String pathResource) {
		
		if (pathResource.startsWith("/ressources") || pathResource.startsWith("/resources")){
			IPath path = Spagic1ToSpagic2Activator.getDefault().getStateLocation();
			System.out.println(" Configuration file for Spagic Studio are located to ["+path.toOSString()+"]");
			
			String cfgDirPath = path.toOSString() + File.separator + "conf"+ File.separator;
        	String resourceName = pathResource.substring(pathResource.lastIndexOf("/") + 1);
        	return "file:"+ cfgDirPath + resourceName;
        }else{
        	// Il Vecchio codice
        	if (pathResource == null) {
        		throw new IllegalArgumentException();
        	}
        	URL url = null;
        	//we use Platform whereas it's depreciated in eclipse 3.2
        	//but FileLocator is not allowed into eclipse 3.1
        	Bundle bundle = Platform.getBundle(PLUGIN_ID);
        	url = FileLocator.find(bundle, new Path(pathResource), null);
        	//url = Platform.find(bundle, new Path(path), null);
        	try {
        		URL fullPath = FileLocator.resolve(url);
        		return fullPath.toString();
        	} catch (IOException e) {
        		SpagicLogger.error("Error in retrieving path"+pathResource , e);
        		return null;
        	}
        }
    }
		
	public static File getFileFromPlugin(String filePath){
		Bundle bundle = getDefault().getBundle();
		Path path = new Path(filePath); 
		URL url = FileLocator.find(bundle, path, Collections.EMPTY_MAP);
		URL fileUrl = null;
		try {
			fileUrl = FileLocator.toFileURL(url);
		}catch (IOException e) {
			// Will happen if the file cannot be read for some reason
			e.printStackTrace();
		}
		return new File(fileUrl.getPath());
	}

}
