package org.spagic.pluginEditor.popupMenu;

import java.io.File;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EcoreFactory;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.stp.bpmn.Activity;
import org.eclipse.stp.bpmn.ActivityType;
import org.eclipse.stp.bpmn.BpmnDiagram;
import org.eclipse.stp.bpmn.BpmnFactory;
import org.eclipse.stp.bpmn.Pool;
import org.eclipse.stp.bpmn.SequenceEdge;
import org.eclipse.stp.bpmn.Vertex;
import org.eclipse.stp.im.constants.ImConstants;
import org.eclipse.stp.im.runtime.IRuntime;
import org.eclipse.stp.im.runtime.IService;
import org.eclipse.stp.im.runtime.ImRuntimeActivator;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.spagic.bpmn.conversion.Mapping;
import org.spagic.bpmn.conversion.Property;
import org.spagic.bpmn.conversion.Spagic1ToSpagic2Activator;
import org.spagic.utils.SpagicLogger;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


import com.bull.cimero.pluginEditor.editors.model.Connection;
import com.bull.cimero.pluginEditor.editors.model.Diagram;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.BCModel.BindingComponentModel;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Routeur;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Routeur.Rule;

/**
 * @author Gianfranco Boccalon (gianfranco.boccalon@eng.it)
 */
public class ConvertToSpagic2 implements IObjectActionDelegate {
	
	private static final String JBI_SMX_RUNTIME = "runtime.jbi.smx";
	
	private static final String ROUTER_COMPONENT = "com.bull.cimero.pluginEditor.editors.model.SEModel.Routeur";
	private static final String SYNCHRONIZER_COMPONENT = "it.eng.components.cimero.GenericSync";
	
	private static final String SERVICE_BINDING = "service-binding";
	private static final String MODEL_CLASS_NAME = "modelClassName";
	private static final String CONDITION_KEY = "key";
	private static final String CONDITION_KEY_VALUES = "keyValues";
	private static final String BINDING_VALUE = "binding";
	private static final String IGNORE_ALL_PROPERTIES = "ignoreAllProperties";

	private static final String ROUTER_CONDITION_EXPRESSION_LANGUAGE_PROPERTY = "ExpressionLanguage";
	private static final String ROUTER_CONDITION_EXPRESSION_LANGUAGE_VALUE = "XPath";
	private static final String ROUTER_CONDITION_XPATH_EXPRESSION_NAME = "Boolean XPath Expression";
	private static final String ROUTER_CONDITION_XPATH_EXPRESSION_PROPERTY = "Expression";
	
	private static final String ROUTER_CONDITION_PROP_EXPRESSION_NAME = "Boolean Expression on Message Properties";
	private static final String ROUTER_CONDITION_PROP_PROPERTY_VALUE = "PropertyValue";
	private static final String ROUTER_CONDITION_PROP_OPERATOR = "Operator";
	private static final String ROUTER_CONDITION_PROP_PROPERTY_NAME = "PropertyName";
	private static final String ROUTER_CONDITION_PROP_EQUALS = "equals";
	private static final String ROUTER_CONDITION_PROP_DIFFERENT = "different";
	

	/** the selected file. */
	private IFile fileSelected = null;
	
	private Map<String, Mapping> mappings;

	/**
	 * Constructor for GenerateJarFileAction.
	 */
	public ConvertToSpagic2() {
		super();
		mappings = new HashMap<String, Mapping>();
	}

	/**
	 * the getter fileselected function.
	 * @return IFile
	 */
	public final IFile getFileSelected() {
		return fileSelected;
	}

	/**
	 * The setter file selected function.
	 * @param filePath : the file
	 */
	public final void setFileSelected(final IFile filePath) {
		this.fileSelected = filePath;
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 * @param action the action proxy that handles the presentation
	 * portion of the action
	 * @param targetPart the new part target
	 */
	public final void setActivePart(final IAction action,
			final IWorkbenchPart targetPart) {
	}

	/**
	 * Read the configuration file and initializes the processors.
	 * @param f The configuration file
	 */
	private void init(File f) {
		try {
			// Parse the XML as a W3C document.
			DocumentBuilder builder = DocumentBuilderFactory.newInstance()
					.newDocumentBuilder();
			Document document = builder.parse(f);
			initMappings(document);
		} catch (Exception ex) {
			SpagicLogger.error(Spagic1ToSpagic2Activator.PLUGIN_ID,"Error reading configuration file", ex);
		}
	}
	
	/**
	 * Instantiates all binding specific processors
	 * @param document Processors configuration DOM
	 * @throws Exception
	 */
	private void initMappings(Document document) throws Exception {
		String xPath = "/mappings/mapping";
		NodeList mappingNodes = selectNodes(document, xPath);
		
		for (int i = 0; i < mappingNodes.getLength(); i++) {
			Node mappingForModelClassName = mappingNodes.item(i);
			Node serviceBindingNode = selectNode(mappingForModelClassName, SERVICE_BINDING);
			String modelClassName = getAttributeValue(mappingForModelClassName, MODEL_CLASS_NAME);
			String key = getAttributeValue(serviceBindingNode, CONDITION_KEY);
			String keyValues = getAttributeValue(serviceBindingNode, CONDITION_KEY_VALUES);
			String binding = getAttributeValue(serviceBindingNode, BINDING_VALUE);
			boolean ignoreAllProperties = "true".equals(getAttributeValue(serviceBindingNode, IGNORE_ALL_PROPERTIES));
			
			Mapping conversionMapping = new Mapping(modelClassName, key, keyValues, binding,
					new HashMap<String, Property>(),
					new HashMap<String, Property>(),
					new HashMap<String, Property>(),
					ignoreAllProperties);
			mappings.put(modelClassName, conversionMapping);
			
			retrieveProperties(mappingForModelClassName, conversionMapping);
		}
	}

	private NodeList selectNodes(Node document, String xPath) throws XPathExpressionException {
		XPath xpath = XPathFactory.newInstance().newXPath();
        return (NodeList)xpath.evaluate(xPath, document, XPathConstants.NODESET);
	}
	
	private Node selectNode(Node document, String xPath) throws XPathExpressionException {
		XPath xpath = XPathFactory.newInstance().newXPath();
        return (Node)xpath.evaluate(xPath, document, XPathConstants.NODE);
	}
	
	private String getAttributeValue(Node n, String attributeName){
		try{
			Node attributeNode = n.getAttributes().getNamedItem(attributeName); 
			return (attributeNode != null ? attributeNode.getNodeValue() : null);
		}catch (Exception e){
			e.printStackTrace();
			return null;
		}
	}
		
	/**
	 * @see IActionDelegate#run(IAction)
	 * @param action the action proxy that handles the presentation
	 * portion of the action
	 */
	public final void run(final IAction action) {
		Shell shell = new Shell();
		if (getFileSelected() == null) {
			//we cannot call the generator without selected file.
			// a message is sent to the user
			MessageDialog.openInformation(
					shell,
					"Spagic 1.1 to Spagic 2.0 conversion Plug-in",
					"There is a problem with the selected file. The conversion cannot be done");
		} else {
			try {
				init(Spagic1ToSpagic2Activator.getFileFromPlugin("conf/mappings.xml"));
				
				
				// attempt to read from a file
				InputStream istream = getFileSelected().getContents(false);
				Diagram diagram = Diagram.makeFromStream(istream);
					
				BpmnDiagram bpmnDiagram = 
					BpmnFactory.eINSTANCE.createBpmnDiagram();

				
				Pool pool = BpmnFactory.eINSTANCE.createPool();
				pool.setName("Test");
				pool.setBpmnDiagram(bpmnDiagram);
				
				bpmnDiagram.getPools().add(pool);
				
				// Create technology annotations for pool
    			EAnnotation imPoolAnnotation = EcoreFactory.eINSTANCE.createEAnnotation();
    			imPoolAnnotation.setSource(ImConstants.TECHNOLOGY_ANNOTATION);
    			pool.getEAnnotations().add(imPoolAnnotation);
    			
    			// Fill annotation
    			imPoolAnnotation.getDetails().put(ImConstants.IM_POOL_RUNTIME_ID, JBI_SMX_RUNTIME);
    			imPoolAnnotation.getDetails().put(ImConstants.TECHNOLOGY_NAME, "JBI - Apache ServiceMix");
				
				List<GeneriqueCimeroModel> listModels = diagram.getLstCimeroObject();
				List<Rule> routerRules = new ArrayList<Rule>();
				List<Connection> incomingSyncConnections = new ArrayList<Connection>();
				List<Connection> outgoingSyncConnections = new ArrayList<Connection>();
				
				for (GeneriqueCimeroModel aModel : listModels) {
					
					String modelClassName = aModel.getClass().getName();
					
					// Create BPMN activity
					Activity bpmnActivity = BpmnFactory.eINSTANCE.createActivity();
					// Set the right type for start and end nodes
					if (aModel instanceof BindingComponentModel) {
	                    ActivityType bpmnActivityType = ((BindingComponentModel)aModel).isInput() ? ActivityType.EVENT_END_EMPTY_LITERAL : ActivityType.EVENT_START_EMPTY_LITERAL;
	                    bpmnActivity.setActivityType(bpmnActivityType);
					}
                    bpmnActivity.setName(aModel.getName());
					pool.getVertices().add(bpmnActivity);

                    // Create new annotation for Intermediate Model
        			EAnnotation imAnnotation = EcoreFactory.eINSTANCE.createEAnnotation();
        			imAnnotation.setSource(ImConstants.IM_ANNOTATION);
        			bpmnActivity.getEAnnotations().add(imAnnotation);
        			
        			
        			// Service binding association
        			Mapping conversionMapping = mappings.get(modelClassName);
        			
					if (conversionMapping == null) {
						SpagicLogger.error(
								"Mapping not defined for Cimero model class name: " + modelClassName);
					}
        			{
	        			// Check if the binding is conditioned to a property
	        			String keyValue = conversionMapping.getKey();
	        			String bindingValue = conversionMapping.getBinding();
        		    	String serviceName = null;
	        			if (keyValue == null) {
	        				// Unconditioned binding
	        			} else {
	        				// Conditioned binding
	        				String[] bindings = bindingValue.split(";");
	        				Object propertyValue = aModel.getPropertyValue(keyValue);

	        				if (conversionMapping.getKeyValues() != null) {
	        					// The Condition key is a String value
	        					String[] keyValues = conversionMapping.getKeyValues().split(";");
	        					
	        					// Search the key within all possible values
	        					List<String> keyValuesList = Arrays.asList(keyValues);
	        					int keyPos = keyValuesList.indexOf(propertyValue);
	        					if (keyPos != -1) {
	        						bindingValue = bindings[keyPos];
	        					} else {
									SpagicLogger.error(
											"Binding not retrievable for modelClassName: " + modelClassName + " and key: " + propertyValue);
	        					}
	        				} else {
	        					// The Condition key is an integer value
		        				// Retrieve the value of the property that conditions
		        				// the binding
		        				Integer propertyConditionValue = (Integer)propertyValue;
		        				
		        				// The condition should be a number
		        				bindingValue = bindings[propertyConditionValue];
	        				}
	        			}
	        			
	        			// Retrieve service name (category) from jbi-runtime.xml
	        			{
	        		        // Retrieve JBI SMX runtime
	        		    	IRuntime jbiRuntime = ImRuntimeActivator.getRuntime(JBI_SMX_RUNTIME);
	        		    	
	        		    	// Search the service name (StartService, EndService, etc) given the specific binding
	        		    	for (IService service : jbiRuntime.getServices()) {
	        		    		if (service.getServiceBindingsName().contains(bindingValue)) {
	        		    			serviceName = service.getName();
	        		    			break;
	        		    		}
	        		    	}
	        			}
	        			
						imAnnotation.getDetails().put(ImConstants.IM_SERVICE_NAME, serviceName);
						imAnnotation.getDetails().put(ImConstants.IM_SERVICE_BINDING_NAME, bindingValue);
						imAnnotation.getDetails().put(ImConstants.IM_POOL_RUNTIME_ID, JBI_SMX_RUNTIME);
        			}
					
					// Map Spagic Studio 1.1 properties to Intermediate Model properties
        			if (!conversionMapping.isIgnoreAllProperties()) {
        				
            			Set<String> propertiesKey = aModel.getPropertyKeys();
    					for (String pKey : propertiesKey) {
    						Object propertyValue = aModel.getPropertyValue(pKey);
    						
    						// Retrieve mapping information
    						Property propertyMapping = conversionMapping.getPropertyMap().get(pKey);
    						
    						if (propertyMapping == null) {
    							
    							// Check if this is an array property
    							Map<String, Property> paramArraysPropertyMap = conversionMapping.getParamArraysPropertyMap();
    							for (String pArrayKey : paramArraysPropertyMap.keySet()) {
    								if (pKey.startsWith(pArrayKey)) {
    									propertyMapping = paramArraysPropertyMap.get(pArrayKey);
    									break;
    								}
    							}
    							
    							if (propertyMapping == null) {
    								SpagicLogger.error(
    										"Mapping not defined for property: " + pKey + " and modelClassName: " + modelClassName);
    							}
    						}
    						
    						// Check if this is a combo
    						String imAnnotationKey = propertyMapping.getImAnnotationKey();
    						String imPropertyValue = null;
    						
    						if (propertyMapping.getPrefixKey() != null) {
    							String realPropertyName = pKey.replaceFirst(propertyMapping.getPrefixKey(), "");
    							imAnnotationKey = propertyMapping.getImParameterArray() + "[" + realPropertyName + "]." + propertyMapping.getImAnnotationKey();
    							
    							if (propertyMapping.getCheckInParameter() != null) {
    								// Consider the property only if its name is contained in another property
    								String containerPropertyValue = (String)aModel.getPropertyValue(propertyMapping.getCheckInParameter());
    								if (containerPropertyValue.indexOf(realPropertyName) == -1) {
    									// Dont use this property
    									imAnnotationKey = "";
    								}
    							}
    						}
    						
    						if (propertyMapping.getImValues() != null) {
    							// It's a combo: should map a numeric value to a String value
    							imPropertyValue = propertyMapping.getImValues()[(Integer)propertyValue];
    						} else {
    							// It's a standard property: can simply copy the value
    							imPropertyValue = (propertyValue != null) ? propertyValue.toString() : null;
    							
    							// Check if we have to remove part of the value
    							if ((imPropertyValue != null) && (propertyMapping.getRemoveStartSubValue() != null)) {
    								imPropertyValue = imPropertyValue.replaceFirst(propertyMapping.getRemoveStartSubValue(), "");
    							}
    						}
    						if (!"".equals(imAnnotationKey)) {
    							imAnnotation.getDetails().put(imAnnotationKey, imPropertyValue);
    						} else {
    							// Property not mapped: not an error
    						}
    					}
        			}
        			
					
					// Copy all fixed values, not dependent on Cimero Model values
					for (Property fixedValueProperty : conversionMapping.getFixedValuesPropertyMap().values()) {
						imAnnotation.getDetails().put(fixedValueProperty.getImAnnotationKey(), fixedValueProperty.getImFixedValue());
					}
					
					
					if (modelClassName.equals(ROUTER_COMPONENT)) {
						// The router has a complex management, so it's not completely mapped in mappings.xml
						Routeur rout = (Routeur)aModel;
						
						// Store all rules in a list for use them when creating the connections
						routerRules.addAll(rout.getMaLstRule());
					}

					if (modelClassName.equals(SYNCHRONIZER_COMPONENT)) {
						// The synchronizer has a complex management, so it's not completely mapped in mappings.xml
						
						// Change activity type
	                    bpmnActivity.setActivityType(ActivityType.EVENT_END_EMPTY_LITERAL);
						
						incomingSyncConnections.addAll(aModel.getIncomingConnections());
						outgoingSyncConnections.addAll(aModel.getOutgoingConnections());
					}
				}
				
				// Create all connections
				for (GeneriqueCimeroModel aModel : listModels) {
					
					List<Connection> incomings = aModel.getIncomingConnections();
					for (Connection aConnection : incomings) {
						String sourceName = aConnection.getSource().getName();
						String targetName = aConnection.getTarget().getName();
						Vertex sourceActivity = findActivity(pool, sourceName);
						
						// Check if this connection is an output connection from the Synchronizer
						if (outgoingSyncConnections.contains(aConnection)) {
							// Skip it: it will be managed indirectly, when managing input
							// Synchronizer connections
							continue;
						}
						
						// Check if this connection is an input connection to the Synchronizer
						if (incomingSyncConnections.contains(aConnection)) {
							
							// Check the start node of the connection
							if (aConnection.getSource().getIncomingConnections().size() == 0) {
								// The activity is an input binding component:
								// change the activity destination to the component
								// that is output of the synchronizer
								targetName = outgoingSyncConnections.get(0).getTarget().getName();
							} else {
								// This connection is an input for the synchronizer:
								// it's at the end of the process and doesnt need special
								// management
							}
						
						}
						
						
						// Create the SequenceEdge
						SequenceEdge sequenceEdge = BpmnFactory.eINSTANCE.createSequenceEdge();
						sequenceEdge.setSource(sourceActivity);
						sequenceEdge.setTarget(findActivity(pool, targetName));
						
						// Check if this connection is related to a router rule
						for (Rule routerRule : routerRules) {
							if (aConnection.equals(routerRule.getConn())) {

			                    // Create new annotation for router rule
			        			EAnnotation ruleAnnotation = EcoreFactory.eINSTANCE.createEAnnotation();
			        			ruleAnnotation.setSource(ImConstants.IM_TRANSITION_ANNOTATION);
			        			sequenceEdge.getEAnnotations().add(ruleAnnotation);
			        			
			        			ruleAnnotation.getDetails().put(ImConstants.IM_POOL_RUNTIME_ID, JBI_SMX_RUNTIME);
			        			
			        			if (!routerRule.isTypeRule()) {
			        				// XPath comparison rule
			        				ruleAnnotation.getDetails().put(ImConstants.IM_TRANSITION_TYPE, ImConstants.IM_TRANSITION_TYPE_EXPRESSION_CONDITION);
			        				ruleAnnotation.getDetails().put(ImConstants.IM_TRANSITION_CONDITION_NAME, ROUTER_CONDITION_XPATH_EXPRESSION_NAME);
			        				
			        				ruleAnnotation.getDetails().put(ROUTER_CONDITION_EXPRESSION_LANGUAGE_PROPERTY, ROUTER_CONDITION_EXPRESSION_LANGUAGE_VALUE);
			        				ruleAnnotation.getDetails().put(ROUTER_CONDITION_XPATH_EXPRESSION_PROPERTY, routerRule.getXPathRule());
			        			} else {
			        				// Content based comparison rule
			        				ruleAnnotation.getDetails().put(ImConstants.IM_TRANSITION_TYPE, ImConstants.IM_TRANSITION_TYPE_PROPERTY_CONDITION);
			        				ruleAnnotation.getDetails().put(ImConstants.IM_TRANSITION_CONDITION_NAME, ROUTER_CONDITION_PROP_EXPRESSION_NAME);

			        				ruleAnnotation.getDetails().put(ROUTER_CONDITION_PROP_PROPERTY_VALUE, routerRule.getValueRule());
			        				ruleAnnotation.getDetails().put(ROUTER_CONDITION_PROP_OPERATOR, routerRule.getTypeComparaison().equals(Routeur.Rule.EGAL) ? ROUTER_CONDITION_PROP_EQUALS : ROUTER_CONDITION_PROP_DIFFERENT);
			        				ruleAnnotation.getDetails().put(ROUTER_CONDITION_PROP_PROPERTY_NAME, routerRule.getValuePropertyName());
			        			}
								break;
							}
						}
						
						pool.getSequenceEdges().add(sequenceEdge);
					}
				}
				
				
				writeBpmnModel(bpmnDiagram);
				
				MessageDialog.openInformation(
						shell,
						"Spagic 1.1 to Spagic 2.0 conversion Plug-in",
						"Conversion from Spagic 1.1 to BPMN was executed.");
			} catch (Exception e) {
				SpagicLogger.error(
						"JAR file generation exception"
						, e);
				MessageDialog.openInformation(
						shell,
						"Spagic 1.1 to Spagic 2.0 conversion Plug-in",
						e.getMessage());
			}
		}
	}
	
	private Vertex findActivity(Pool pool, String name) {
		for (Vertex aVertex : pool.getVertices()) {
			if ((aVertex instanceof Activity) && (((Activity)aVertex).getName().equals(name))) {
				return (Activity)aVertex;
			}
		}
		return null;
	}
	
	private void retrieveProperties(final Node mappingForModelClassName, Mapping conversionMapping) throws XPathExpressionException {
		NodeList propertiesMapping = selectNodes(mappingForModelClassName, "property");
		for (int i = 0; i < propertiesMapping.getLength(); i++) {
			Node propertyMapping = propertiesMapping.item(i);
			
			String key = getAttributeValue(propertyMapping, "key");
			String imAnnotationKey = getAttributeValue(propertyMapping, "imAnnotationKey");
			String imValues = getAttributeValue(propertyMapping, "imValues");
			String[] imValuesArray = (imValues != null) ? imValues.split(";") : null;
			String imFixedValue = getAttributeValue(propertyMapping, "imFixedValue");
			String checkInParameter = getAttributeValue(propertyMapping, "checkInParameter");
			
			String prefixKey = getAttributeValue(propertyMapping, "prefixKey");
			String imParameterArray = getAttributeValue(propertyMapping, "imParameterArray");
			String removeStartSubValue = getAttributeValue(propertyMapping, "removeStartSubValue");
			
			Property property = new Property(key, imAnnotationKey, imValuesArray, imFixedValue, prefixKey, imParameterArray, checkInParameter, removeStartSubValue);
			if (prefixKey != null) {
				// Array property
				conversionMapping.getParamArraysPropertyMap().put(property.getPrefixKey(), property);				
			} else if (key != null) {
				// Non fixed value
				conversionMapping.getPropertyMap().put(key, property);
			} else {
				// Fixed value
				conversionMapping.getFixedValuesPropertyMap().put(property.getImAnnotationKey(), property);
			}
		}
	}
	
	
	private void writeBpmnModel(BpmnDiagram aBpmnModel) throws Exception {
		IContainer container = getFileSelected().getParent();

		// Create a resource set.
		ResourceSet resourceSetStpIm = new ResourceSetImpl();
		// Get the URI of the model file.
		String destinationBpmnFileName = getFileSelected().getName();
		destinationBpmnFileName = destinationBpmnFileName.substring(0,
				destinationBpmnFileName.indexOf("."));

		destinationBpmnFileName = destinationBpmnFileName + ".bpmn";

		IFile destinationStpFile = container.getFile(new Path(
				destinationBpmnFileName));
		URI destinationStpFileURI = URI.createFileURI(destinationStpFile
				.getFullPath().toString());

		// Create a resource for this file.
		Resource resourceBpmn = resourceSetStpIm
				.createResource(destinationStpFileURI);
		resourceBpmn.getContents().add(aBpmnModel);

		resourceBpmn.save(Collections.EMPTY_MAP);
	}
	

	/**
	 * This function is called when the user selected the file in the package
	 * explorer.
	 * 
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 * @param action
	 *            the action proxy that handles the presentation portion of the
	 *            action
	 * @param selection :
	 *            the selected file
	 */
	public final void selectionChanged(final IAction action,
			final ISelection selection) {
		//we save the select file
		setFileSelected((IFile) ((StructuredSelection) selection)
				.getFirstElement());
	}
}
